@@ -43,3 +43,31 @@ def conv_sequences(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> Sequence[H
         return s1, s2
 
     return conv_sequence(s1), conv_sequence(s2)
+
+
+def common_prefix(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> int:
+    prefix_len = 0
+    for ch1, ch2 in zip(s1, s2):
+        if ch1 != ch2:
+            break
+
+        prefix_len += 1
+
+    return prefix_len
+
+
+def common_suffix(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> int:
+    suffix_len = 0
+    for ch1, ch2 in zip(reversed(s1), reversed(s2)):
+        if ch1 != ch2:
+            break
+
+        suffix_len += 1
+
+    return suffix_len
+
+
+def common_affix(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> (int, int):
+    prefix_len = common_prefix(s1, s2)
+    suffix_len = common_suffix(s1[prefix_len:], s2[prefix_len:])
+    return (prefix_len, suffix_len)
