@@ -5,14 +5,13 @@ from __future__ import annotations
 
 from typing import Callable, Hashable, Sequence
 
-from rapidfuzz.distance import Indel
-from rapidfuzz.distance._initialize import Editops, Opcodes
+from rapidfuzz._common_py import common_affix, conv_sequences
 from rapidfuzz._utils import is_none
+from rapidfuzz.distance import Indel_py as Indel
+from rapidfuzz.distance._initialize_py import Editop, Editops, Opcodes
 
 
-def _levenshtein_maximum(
-    s1: Sequence[Hashable], s2: Sequence[Hashable], weights: tuple[int, int, int]
-) -> int:
+def _levenshtein_maximum(s1: Sequence[Hashable], s2: Sequence[Hashable], weights: tuple[int, int, int]) -> int:
     len1 = len(s1)
     len2 = len(s2)
     insert, delete, replace = weights
@@ -27,9 +26,7 @@ def _levenshtein_maximum(
     return max_dist
 
 
-def _uniform_generic(
-    s1: Sequence[Hashable], s2: Sequence[Hashable], weights: tuple[int, int, int]
-) -> int:
+def _uniform_generic(s1: Sequence[Hashable], s2: Sequence[Hashable], weights: tuple[int, int, int]) -> int:
     len1 = len(s1)
     insert, delete, replace = weights
     cache = list(range(0, (len1 + 1) * delete, delete))
@@ -149,10 +146,12 @@ def distance(
     >>> Levenshtein.distance("lewenstein", "levenshtein", weights=(1,1,2))
     3
     """
+    _ = score_hint
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
 
+    s1, s2 = conv_sequences(s1, s2)
     if weights is None or weights == (1, 1, 1):
         dist = _uniform_distance(s1, s2)
     elif weights == (1, 1, 2):
@@ -211,10 +210,12 @@ def similarity(
     ValueError
         If unsupported weights are provided a ValueError is thrown
     """
+    _ = score_hint
     if processor is not None:
         s1 = processor(s1)
         s2 = processor(s2)
 
+    s1, s2 = conv_sequences(s1, s2)
     weights = weights or (1, 1, 1)
     maximum = _levenshtein_maximum(s1, s2, weights)
     dist = distance(s1, s2, weights=weights)
@@ -269,6 +270,7 @@ def normalized_distance(
     ValueError
         If unsupported weights are provided a ValueError is thrown
     """
+    _ = score_hint
     if is_none(s1) or is_none(s2):
         return 1.0
 
@@ -276,6 +278,7 @@ def normalized_distance(
         s1 = processor(s1)
         s2 = processor(s2)
 
+    s1, s2 = conv_sequences(s1, s2)
     weights = weights or (1, 1, 1)
     maximum = _levenshtein_maximum(s1, s2, weights)
     dist = distance(s1, s2, weights=weights)
@@ -354,6 +357,7 @@ def normalized_similarity(
     >>> Levenshtein.normalized_similarity(["lewenstein"], ["levenshtein"], processor=lambda s: s[0])
     0.81818181818181
     """
+    _ = score_hint
     if is_none(s1) or is_none(s2):
         return 0.0
 
@@ -361,12 +365,54 @@ def normalized_similarity(
         s1 = processor(s1)
         s2 = processor(s2)
 
+    s1, s2 = conv_sequences(s1, s2)
     weights = weights or (1, 1, 1)
     norm_dist = normalized_distance(s1, s2, weights=weights)
     norm_sim = 1.0 - norm_dist
     return norm_sim if (score_cutoff is None or norm_sim >= score_cutoff) else 0
 
 
+def _matrix(s1: Sequence[Hashable], s2: Sequence[Hashable]) -> int:
+    if not s1:
+        return (len(s2), [], [])
+
+    VP = (1 << len(s1)) - 1
+    VN = 0
+    currDist = len(s1)
+    mask = 1 << (len(s1) - 1)
+
+    block: dict[Hashable, int] = {}
+    block_get = block.get
+    x = 1
+    for ch1 in s1:
+        block[ch1] = block_get(ch1, 0) | x
+        x <<= 1
+
+    matrix_VP = []
+    matrix_VN = []
+    for ch2 in s2:
+        # Step 1: Computing D0
+        PM_j = block_get(ch2, 0)
+        X = PM_j
+        D0 = (((X & VP) + VP) ^ VP) | X | VN
+        # Step 2: Computing HP and HN
+        HP = VN | ~(D0 | VP)
+        HN = D0 & VP
+        # Step 3: Computing the value D[m,j]
+        currDist += (HP & mask) != 0
+        currDist -= (HN & mask) != 0
+        # Step 4: Computing Vp and VN
+        HP = (HP << 1) | 1
+        HN = HN << 1
+        VP = HN | ~(D0 | HP)
+        VN = HP & D0
+
+        matrix_VP.append(VP)
+        matrix_VN.append(VN)
+
+    return (currDist, matrix_VP, matrix_VN)
+
+
 def editops(
     s1: Sequence[Hashable],
     s2: Sequence[Hashable],
@@ -414,7 +460,60 @@ def editops(
     replace s1[3] s2[2]
      insert s1[6] s2[5]
     """
-    raise NotImplementedError
+    _ = score_hint
+    if processor is not None:
+        s1 = processor(s1)
+        s2 = processor(s2)
+
+    s1, s2 = conv_sequences(s1, s2)
+    prefix_len, suffix_len = common_affix(s1, s2)
+    s1 = s1[prefix_len : len(s1) - suffix_len]
+    s2 = s2[prefix_len : len(s2) - suffix_len]
+    dist, VP, VN = _matrix(s1, s2)
+
+    editops = Editops([], 0, 0)
+    editops._src_len = len(s1) + prefix_len + suffix_len
+    editops._dest_len = len(s2) + prefix_len + suffix_len
+
+    if dist == 0:
+        return editops
+
+    editop_list = [None] * dist
+    col = len(s1)
+    row = len(s2)
+    while row != 0 and col != 0:
+        # deletion
+        if VP[row - 1] & (1 << (col - 1)):
+            dist -= 1
+            col -= 1
+            editop_list[dist] = Editop("delete", col + prefix_len, row + prefix_len)
+        else:
+            row -= 1
+
+            # insertion
+            if row and (VN[row - 1] & (1 << (col - 1))):
+                dist -= 1
+                editop_list[dist] = Editop("insert", col + prefix_len, row + prefix_len)
+            else:
+                col -= 1
+
+                # replace (Matches are not recorded)
+                if s1[col] != s2[row]:
+                    dist -= 1
+                    editop_list[dist] = Editop("replace", col + prefix_len, row + prefix_len)
+
+    while col != 0:
+        dist -= 1
+        col -= 1
+        editop_list[dist] = Editop("delete", col + prefix_len, row + prefix_len)
+
+    while row != 0:
+        dist -= 1
+        row -= 1
+        editop_list[dist] = Editop("insert", col + prefix_len, row + prefix_len)
+
+    editops._editops = editop_list
+    return editops
 
 
 def opcodes(
@@ -470,4 +569,4 @@ def opcodes(
       equal a[4:6] (cd) b[3:5] (cd)
      insert a[6:6] () b[5:6] (f)
     """
-    raise NotImplementedError
+    return editops(s1, s2, processor=processor, score_hint=score_hint).as_opcodes()
